@@ -5,31 +5,43 @@
 use super::{MutableZeroVecLike, ZeroMap, ZeroMapBorrowed, ZeroMapKV, ZeroVecLike};
 use core::fmt;
 use core::marker::PhantomData;
-use serde::de::{self, MapAccess, Visitor};
-use serde::ser::SerializeMap;
-use serde::{Deserialize, Deserializer, Serialize, Serializer};
+use serde::de::{self, Deserialize, Deserializer, MapAccess, SeqAccess, Visitor};
+#[cfg(feature = "serde_serialize")]
+use serde::ser::{Serialize, SerializeMap, SerializeSeq, Serializer};
 
-/// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
+/// This impl can be made available by enabling the optional `serde_serialize` feature of the `zerovec` crate
+#[cfg(feature = "serde_serialize")]
 impl<'a, K, V> Serialize for ZeroMap<'a, K, V>
 where
-    K: ZeroMapKV<'a>,
-    V: ZeroMapKV<'a>,
-    K: ?Sized,
-    V: ?Sized,
+    K: ZeroMapKV<'a> + Serialize + ?Sized + Ord,
+    V: ZeroMapKV<'a> + Serialize + ?Sized,
     K::Container: Serialize,
     V::Container: Serialize,
-    K: Serialize,
-    V: Serialize,
 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
         if serializer.is_human_readable() {
+            // Many human-readable formats don't support values other
+            // than numbers and strings as map keys. For them, we can serialize
+            // as a vec of tuples instead
+            if let Some(k) = self.iter_keys().next() {
+                let json = K::Container::zvl_get_as_t(k, |k| serde_json::json!(k));
+                if !json.is_string() && !json.is_number() {
+                    let mut seq = serializer.serialize_seq(Some(self.len()))?;
+                    for (k, v) in self.iter() {
+                        K::Container::zvl_get_as_t(k, |k| {
+                            V::Container::zvl_get_as_t(v, |v| seq.serialize_element(&(k, v)))
+                        })?;
+                    }
+                    return seq.end();
+                }
+            }
             let mut map = serializer.serialize_map(Some(self.len()))?;
             for (k, v) in self.iter() {
-                K::Container::t_with_ser(k, |k| map.serialize_key(k))?;
-                V::Container::t_with_ser(v, |v| map.serialize_value(v))?;
+                K::Container::zvl_get_as_t(k, |k| map.serialize_key(k))?;
+                V::Container::zvl_get_as_t(v, |v| map.serialize_value(v))?;
             }
             map.end()
         } else {
@@ -38,17 +50,14 @@ where
     }
 }
 
-/// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
+/// This impl can be made available by enabling the optional `serde_serialize` feature of the `zerovec` crate
+#[cfg(feature = "serde_serialize")]
 impl<'a, K, V> Serialize for ZeroMapBorrowed<'a, K, V>
 where
-    K: ZeroMapKV<'a>,
-    V: ZeroMapKV<'a>,
-    K: ?Sized,
-    V: ?Sized,
+    K: ZeroMapKV<'a> + Serialize + ?Sized + Ord,
+    V: ZeroMapKV<'a> + Serialize + ?Sized,
     K::Container: Serialize,
     V::Container: Serialize,
-    K: Serialize,
-    V: Serialize,
 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
@@ -59,12 +68,20 @@ where
 }
 
 /// Modified example from https://serde.rs/deserialize-map.html
-struct ZeroMapMapVisitor<'a, K: ZeroMapKV<'a> + ?Sized, V: ZeroMapKV<'a> + ?Sized> {
+struct ZeroMapMapVisitor<'a, K, V>
+where
+    K: ZeroMapKV<'a> + ?Sized + Ord,
+    V: ZeroMapKV<'a> + ?Sized,
+{
     #[allow(clippy::type_complexity)] // it's a marker type, complexity doesn't matter
     marker: PhantomData<fn() -> (&'a K::OwnedType, &'a V::OwnedType)>,
 }
 
-impl<'a, K: ZeroMapKV<'a> + ?Sized, V: ZeroMapKV<'a> + ?Sized> ZeroMapMapVisitor<'a, K, V> {
+impl<'a, K, V> ZeroMapMapVisitor<'a, K, V>
+where
+    K: ZeroMapKV<'a> + ?Sized + Ord,
+    V: ZeroMapKV<'a> + ?Sized,
+{
     fn new() -> Self {
         ZeroMapMapVisitor {
             marker: PhantomData,
@@ -72,11 +89,10 @@ impl<'a, K: ZeroMapKV<'a> + ?Sized, V: ZeroMapKV<'a> + ?Sized> ZeroMapMapVisitor
     }
 }
 
-impl<'a, 'de, K: ?Sized, V: ?Sized> Visitor<'de> for ZeroMapMapVisitor<'a, K, V>
+impl<'a, 'de, K, V> Visitor<'de> for ZeroMapMapVisitor<'a, K, V>
 where
-    K: Ord,
-    K: ZeroMapKV<'a>,
-    V: ZeroMapKV<'a>,
+    K: ZeroMapKV<'a> + Ord + ?Sized,
+    V: ZeroMapKV<'a> + ?Sized,
     K::OwnedType: Deserialize<'de>,
     V::OwnedType: Deserialize<'de>,
 {
@@ -86,6 +102,34 @@ where
         formatter.write_str("a map produced by ZeroMap")
     }
 
+    fn visit_seq<S>(self, mut access: S) -> Result<Self::Value, S::Error>
+    where
+        S: SeqAccess<'de>,
+    {
+        let mut map = ZeroMap::with_capacity(access.size_hint().unwrap_or(0));
+
+        // While there are entries remaining in the input, add them
+        // into our map.
+        while let Some((key, value)) = access.next_element::<(K::OwnedType, V::OwnedType)>()? {
+            // Try to append it at the end, hoping for a sorted map.
+            // If not sorted, return an error
+            // a serialized map that came from another ZeroMap
+            if map
+                .try_append(
+                    K::Container::owned_as_t(&key),
+                    V::Container::owned_as_t(&value),
+                )
+                .is_some()
+            {
+                return Err(de::Error::custom(
+                    "ZeroMap's keys must be sorted while deserializing",
+                ));
+            }
+        }
+
+        Ok(map)
+    }
+
     fn visit_map<M>(self, mut access: M) -> Result<Self::Value, M::Error>
     where
         M: MapAccess<'de>,
@@ -116,13 +160,12 @@ where
 }
 
 /// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
-impl<'de, 'a, K: ?Sized, V: ?Sized> Deserialize<'de> for ZeroMap<'a, K, V>
+impl<'de, 'a, K, V> Deserialize<'de> for ZeroMap<'a, K, V>
 where
-    K: Ord,
+    K: ZeroMapKV<'a> + Ord + ?Sized,
+    V: ZeroMapKV<'a> + ?Sized,
     K::Container: Deserialize<'de>,
     V::Container: Deserialize<'de>,
-    K: ZeroMapKV<'a>,
-    V: ZeroMapKV<'a>,
     K::OwnedType: Deserialize<'de>,
     V::OwnedType: Deserialize<'de>,
     'de: 'a,
@@ -132,31 +175,29 @@ where
         D: Deserializer<'de>,
     {
         if deserializer.is_human_readable() {
-            deserializer.deserialize_map(ZeroMapMapVisitor::<'a, K, V>::new())
+            deserializer.deserialize_any(ZeroMapMapVisitor::<'a, K, V>::new())
         } else {
             let (keys, values): (K::Container, V::Container) =
                 Deserialize::deserialize(deserializer)?;
-            if keys.len() != values.len() {
+            if keys.zvl_len() != values.zvl_len() {
                 return Err(de::Error::custom(
                     "Mismatched key and value sizes in ZeroMap",
                 ));
             }
-            if !keys.is_ascending() {
-                return Err(de::Error::custom("ZeroMap deserializing keys out of order"));
-            }
+            // #1433: If keys are out of order, treat it as GIGO.
+            debug_assert!(keys.zvl_is_ascending());
             Ok(Self { keys, values })
         }
     }
 }
 
 // /// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
-impl<'de, 'a, K: ?Sized, V: ?Sized> Deserialize<'de> for ZeroMapBorrowed<'a, K, V>
+impl<'de, 'a, K, V> Deserialize<'de> for ZeroMapBorrowed<'a, K, V>
 where
-    K: Ord,
+    K: ZeroMapKV<'a> + Ord + ?Sized,
+    V: ZeroMapKV<'a> + ?Sized,
     K::Container: Deserialize<'de>,
     V::Container: Deserialize<'de>,
-    K: ZeroMapKV<'a>,
-    V: ZeroMapKV<'a>,
     K::OwnedType: Deserialize<'de>,
     V::OwnedType: Deserialize<'de>,
     'de: 'a,
@@ -171,14 +212,14 @@ where
             ))
         } else {
             let deserialized: ZeroMap<'a, K, V> = ZeroMap::deserialize(deserializer)?;
-            let keys = if let Some(keys) = deserialized.keys.as_borrowed_inner() {
+            let keys = if let Some(keys) = deserialized.keys.zvl_as_borrowed_inner() {
                 keys
             } else {
                 return Err(de::Error::custom(
                     "ZeroMapBorrowed can only deserialize in zero-copy ways",
                 ));
             };
-            let values = if let Some(values) = deserialized.values.as_borrowed_inner() {
+            let values = if let Some(values) = deserialized.values.zvl_as_borrowed_inner() {
                 values
             } else {
                 return Err(de::Error::custom(
@@ -193,15 +234,15 @@ where
 #[cfg(test)]
 #[allow(non_camel_case_types)]
 mod test {
-    use super::super::*;
+    use crate::{map::ZeroMapBorrowed, ZeroMap};
 
-    #[derive(::serde::Serialize, ::serde::Deserialize)]
+    #[derive(serde::Serialize, serde::Deserialize)]
     struct DeriveTest_ZeroMap<'data> {
         #[serde(borrow)]
         _data: ZeroMap<'data, str, [u8]>,
     }
 
-    #[derive(::serde::Serialize, ::serde::Deserialize)]
+    #[derive(serde::Serialize, serde::Deserialize)]
     struct DeriveTest_ZeroMapBorrowed<'data> {
         #[serde(borrow)]
         _data: ZeroMapBorrowed<'data, str, [u8]>,
@@ -220,6 +261,7 @@ mod test {
         map.insert(&3, "tres");
         map
     }
+
     #[test]
     fn test_serde_json() {
         let map = make_map();
@@ -232,6 +274,25 @@ mod test {
         );
     }
 
+    #[test]
+    fn test_serde_json_complex_key() {
+        let mut map = ZeroMap::new();
+        map.insert(&(1, 1), "uno");
+        map.insert(&(2, 2), "dos");
+        map.insert(&(3, 3), "tres");
+        let json_str = serde_json::to_string(&map).expect("serialize");
+        assert_eq!(
+            json_str,
+            "[[[1,1],\"uno\"],[[2,2],\"dos\"],[[3,3],\"tres\"]]"
+        );
+        let new_map: ZeroMap<(u32, u32), str> =
+            serde_json::from_str(&json_str).expect("deserialize");
+        assert_eq!(
+            new_map.iter().collect::<Vec<_>>(),
+            map.iter().collect::<Vec<_>>()
+        );
+    }
+
     #[test]
     fn test_bincode() {
         let map = make_map();
