@@ -6,6 +6,7 @@
 //! ULE implementation for the `char` type.
 
 use super::*;
+use core::cmp::Ordering;
 use core::convert::TryFrom;
 
 /// A u8 array of little-endian data corresponding to a Unicode code point.
@@ -21,7 +22,7 @@ use core::convert::TryFrom;
 /// use zerovec::ule::{ULE, AsULE, CharULE};
 ///
 /// let c1 = '𑄃';
-/// let ule = c1.as_unaligned();
+/// let ule = c1.to_unaligned();
 /// assert_eq!(CharULE::as_byte_slice(&[ule]), &[0x03, 0x11, 0x01, 0x00]);
 /// let c2 = char::from_unaligned(ule);
 /// assert_eq!(c1, c2);
@@ -49,21 +50,18 @@ pub struct CharULE([u8; 4]);
 //  5. The other ULE methods use the default impl.
 //  6. CharULE byte equality is semantic equality
 unsafe impl ULE for CharULE {
-    type Error = ULEError<core::char::CharTryFromError>;
-
     #[inline]
-    fn validate_byte_slice(bytes: &[u8]) -> Result<(), Self::Error> {
+    fn validate_byte_slice(bytes: &[u8]) -> Result<(), ZeroVecError> {
         if bytes.len() % 4 != 0 {
-            return Err(ULEError::InvalidLength {
-                ty: "char",
-                len: bytes.len(),
-            });
+            return Err(ZeroVecError::length::<Self>(bytes.len()));
         }
         // Validate the bytes
         for chunk in bytes.chunks_exact(4) {
             // TODO: Use slice::as_chunks() when stabilized
+            #[allow(clippy::indexing_slicing)]
+            // TODO(#1668) Clippy exceptions need docs or fixing.i
             let u = u32::from_le_bytes([chunk[0], chunk[1], chunk[2], chunk[3]]);
-            char::try_from(u)?;
+            char::try_from(u).map_err(|_| ZeroVecError::parse::<Self>())?;
         }
         Ok(())
     }
@@ -73,7 +71,7 @@ impl AsULE for char {
     type ULE = CharULE;
 
     #[inline]
-    fn as_unaligned(self) -> Self::ULE {
+    fn to_unaligned(self) -> Self::ULE {
         let u = u32::from(self);
         CharULE(u.to_le_bytes())
     }
@@ -83,6 +81,7 @@ impl AsULE for char {
         let u = u32::from_le_bytes(unaligned.0);
         // Safe because the bytes of CharULE are defined to represent a valid Unicode code point.
         // TODO: Use char::from_u32_unchecked() when stabilized
+        #[allow(clippy::unwrap_used)] // TODO(#1668) Clippy exceptions need docs or fixing.
         Self::try_from(u).unwrap()
     }
 }
@@ -91,6 +90,18 @@ impl AsULE for char {
 // as CharULE on little-endian platforms.
 unsafe impl EqULE for char {}
 
+impl PartialOrd for CharULE {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        char::from_unaligned(*self).partial_cmp(&char::from_unaligned(*other))
+    }
+}
+
+impl Ord for CharULE {
+    fn cmp(&self, other: &Self) -> Ordering {
+        char::from_unaligned(*self).cmp(&char::from_unaligned(*other))
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
@@ -99,7 +110,7 @@ mod test {
     fn test_parse() {
         // 1-byte, 2-byte, 3-byte, and 4-byte character in UTF-8 (not as relevant in UTF-32)
         let chars = ['w', 'ω', '文', '𑄃'];
-        let char_ules: Vec<CharULE> = chars.iter().copied().map(char::as_unaligned).collect();
+        let char_ules: Vec<CharULE> = chars.iter().copied().map(char::to_unaligned).collect();
         let char_bytes: &[u8] = CharULE::as_byte_slice(&char_ules);
 
         // Check parsing
@@ -113,7 +124,7 @@ mod test {
         assert_eq!(&chars, parsed_chars.as_slice());
 
         // Check EqULE
-        let char_ule_slice = char::slice_as_unaligned(&chars);
+        let char_ule_slice = char::slice_to_unaligned(&chars);
         #[cfg(target_endian = "little")]
         assert_eq!(char_ule_slice, Some(char_ules.as_slice()));
         #[cfg(not(target_endian = "little"))]
@@ -121,12 +132,12 @@ mod test {
 
         // Compare to u32
         let u32s: Vec<u32> = chars.iter().copied().map(u32::from).collect();
-        let u32_ules: Vec<PlainOldULE<4>> = u32s
+        let u32_ules: Vec<RawBytesULE<4>> = u32s
             .iter()
             .copied()
-            .map(<u32 as AsULE>::as_unaligned)
+            .map(<u32 as AsULE>::to_unaligned)
             .collect();
-        let u32_bytes: &[u8] = PlainOldULE::<4>::as_byte_slice(&u32_ules);
+        let u32_bytes: &[u8] = RawBytesULE::<4>::as_byte_slice(&u32_ules);
         assert_eq!(char_bytes, u32_bytes);
 
         // Compare to golden expected data
@@ -140,23 +151,23 @@ mod test {
     fn test_failures() {
         // 119 and 120 are valid, but not 0xD800 (high surrogate)
         let u32s = [119, 0xD800, 120];
-        let u32_ules: Vec<PlainOldULE<4>> = u32s
+        let u32_ules: Vec<RawBytesULE<4>> = u32s
             .iter()
             .copied()
-            .map(<u32 as AsULE>::as_unaligned)
+            .map(<u32 as AsULE>::to_unaligned)
             .collect();
-        let u32_bytes: &[u8] = PlainOldULE::<4>::as_byte_slice(&u32_ules);
+        let u32_bytes: &[u8] = RawBytesULE::<4>::as_byte_slice(&u32_ules);
         let parsed_ules_result = CharULE::parse_byte_slice(u32_bytes);
         assert!(matches!(parsed_ules_result, Err(_)));
 
         // 0x20FFFF is out of range for a char
         let u32s = [0x20FFFF];
-        let u32_ules: Vec<PlainOldULE<4>> = u32s
+        let u32_ules: Vec<RawBytesULE<4>> = u32s
             .iter()
             .copied()
-            .map(<u32 as AsULE>::as_unaligned)
+            .map(<u32 as AsULE>::to_unaligned)
             .collect();
-        let u32_bytes: &[u8] = PlainOldULE::<4>::as_byte_slice(&u32_ules);
+        let u32_bytes: &[u8] = RawBytesULE::<4>::as_byte_slice(&u32_ules);
         let parsed_ules_result = CharULE::parse_byte_slice(u32_bytes);
         assert!(matches!(parsed_ules_result, Err(_)));
     }
