@@ -2,22 +2,21 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
+use super::*;
+use crate::ule::*;
 use alloc::boxed::Box;
 use alloc::vec::Vec;
-
-use super::*;
-use core::fmt;
 use core::marker::PhantomData;
+use core::ops::Deref;
 use core::ops::Range;
-use core::ptr;
-use core::slice;
+use core::{fmt, ptr, slice};
 
 /// A fully-owned [`VarZeroVec`]. This type has no lifetime but has the same
 /// internal buffer representation of [`VarZeroVec`], making it cheaply convertible to
-/// [`VarZeroVec`] and [`VarZeroVecBorrowed`].
+/// [`VarZeroVec`] and [`VarZeroSlice`].
 pub struct VarZeroVecOwned<T: ?Sized> {
     marker: PhantomData<Box<T>>,
-    // safety invariant: must parse into a valid VarZeroVecBorrowed
+    // safety invariant: must parse into a valid VarZeroVecComponents
     entire_slice: Vec<u8>,
 }
 
@@ -38,6 +37,13 @@ enum ShiftType {
     Remove,
 }
 
+impl<T: VarULE + ?Sized> Deref for VarZeroVecOwned<T> {
+    type Target = VarZeroSlice<T>;
+    fn deref(&self) -> &VarZeroSlice<T> {
+        self.as_slice()
+    }
+}
+
 impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
     /// Construct an empty VarZeroVecOwned
     pub fn new() -> Self {
@@ -47,22 +53,35 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
         }
     }
 
-    /// Construct a VarZeroVecOwned from a [`VarZeroVecBorrowed`] by cloning the internal data
-    pub fn from_borrowed(borrowed: VarZeroVecBorrowed<T>) -> Self {
+    /// Construct a VarZeroVecOwned from a [`VarZeroSlice`] by cloning the internal data
+    pub fn from_slice(slice: &VarZeroSlice<T>) -> Self {
         Self {
             marker: PhantomData,
-            entire_slice: borrowed.entire_slice().into(),
+            entire_slice: slice.as_bytes().into(),
         }
     }
 
     /// Construct a VarZeroVecOwned from a list of elements
-    pub fn from_elements<A: custom::EncodeAsVarULE<T>>(elements: &[A]) -> Self {
-        Self {
+    pub fn try_from_elements<A>(elements: &[A]) -> Result<Self, &'static str>
+    where
+        A: EncodeAsVarULE<T>,
+    {
+        Ok(Self {
             marker: PhantomData,
-            entire_slice: borrowed::get_serializable_bytes(elements).expect(
+            // TODO(#1410): Rethink length errors in VZV.
+            entire_slice: components::get_serializable_bytes(elements).ok_or(
                 "Attempted to build VarZeroVec out of elements that \
                                      cumulatively are larger than a u32 in size",
-            ),
+            )?,
+        })
+    }
+
+    /// Obtain this `VarZeroVec` as a [`VarZeroSlice`]
+    pub fn as_slice(&self) -> &VarZeroSlice<T> {
+        let slice: &[u8] = &*self.entire_slice;
+        unsafe {
+            // safety: the slice is known to come from a valid parsed VZV
+            VarZeroSlice::from_byte_slice_unchecked(slice)
         }
     }
 
@@ -83,41 +102,12 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
         self.entire_slice.reserve(capacity * 8)
     }
 
-    /// Obtain a [`VarZeroVecBorrowed`] borrowing from the internal buffer
-    #[inline]
-    pub fn as_borrowed<'a>(&'a self) -> VarZeroVecBorrowed<'a, T> {
-        unsafe {
-            // safety: VarZeroVecOwned is guaranteed to parse here
-            VarZeroVecBorrowed::from_bytes_unchecked(&self.entire_slice)
-        }
-    }
-
-    /// Get the number of elements in this vector
-    pub fn len(&self) -> usize {
-        self.as_borrowed().len()
-    }
-
-    /// Returns `true` if the vector contains no elements.
-    pub fn is_empty(&self) -> bool {
-        self.as_borrowed().is_empty()
-    }
-
-    /// Obtain an iterator over VarZeroVecOwned's elements
-    pub fn iter<'b>(&'b self) -> impl Iterator<Item = &'b T> {
-        self.as_borrowed().iter()
-    }
-
-    /// Get one of VarZeroVecOwned's elements, returning None if the index is out of bounds
-    pub fn get(&self, idx: usize) -> Option<&T> {
-        self.as_borrowed().get(idx)
-    }
-
     /// Get the position of a specific element in the data segment.
     ///
     /// If `idx == self.len()`, it will return the size of the data segment (where a new element would go).
     ///
     /// ## Safety
-    /// `idx <= self.len()` and `self.entire_slice()` is well-formed.
+    /// `idx <= self.len()` and `self.as_encoded_bytes()` is well-formed.
     unsafe fn element_position_unchecked(&self, idx: usize) -> usize {
         let len = self.len();
         let out = if idx == len {
@@ -132,7 +122,7 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
     /// Get the range of a specific element in the data segment.
     ///
     /// ## Safety
-    /// `idx < self.len()` and `self.entire_slice()` is well-formed.
+    /// `idx < self.len()` and `self.as_encoded_bytes()` is well-formed.
     unsafe fn element_range_unchecked(&self, idx: usize) -> core::ops::Range<usize> {
         let start = self.element_position_unchecked(idx);
         let end = self.element_position_unchecked(idx + 1);
@@ -143,9 +133,10 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
     /// Set the number of elements in the list without any checks.
     ///
     /// ## Safety
-    /// No safe functions may be called until `self.entire_slice()` is well-formed.
+    /// No safe functions may be called until `self.as_encoded_bytes()` is well-formed.
+    #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
     unsafe fn set_len(&mut self, len: u32) {
-        PlainOldULE::<4>::from_byte_slice_unchecked_mut(&mut self.entire_slice[..4])[0] =
+        RawBytesULE::<4>::from_byte_slice_unchecked_mut(&mut self.entire_slice[..4])[0] =
             len.into();
     }
 
@@ -157,18 +148,19 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
     /// Return the slice representing the given `index`.
     ///
     /// ## Safety
-    /// The index must be valid, and self.entire_slice() must be well-formed
-    unsafe fn index_data(&self, index: usize) -> &PlainOldULE<4> {
-        &PlainOldULE::<4>::from_byte_slice_unchecked(&self.entire_slice[Self::index_range(index)])
+    /// The index must be valid, and self.as_encoded_bytes() must be well-formed
+    #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
+    unsafe fn index_data(&self, index: usize) -> &RawBytesULE<4> {
+        &RawBytesULE::<4>::from_byte_slice_unchecked(&self.entire_slice[Self::index_range(index)])
             [0]
     }
 
     /// Return the mutable slice representing the given `index`.
     ///
     /// ## Safety
-    /// The index must be valid. self.entire_slice() must have allocated space
+    /// The index must be valid. self.as_encoded_bytes() must have allocated space
     /// for this index, but need not have its length appropriately set.
-    unsafe fn index_data_mut(&mut self, index: usize) -> &mut PlainOldULE<4> {
+    unsafe fn index_data_mut(&mut self, index: usize) -> &mut RawBytesULE<4> {
         let ptr = self.entire_slice.as_mut_ptr();
         let range = Self::index_range(index);
 
@@ -177,7 +169,8 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
         // if we know the buffer is larger.
         let data = slice::from_raw_parts_mut(ptr.add(range.start), 4);
 
-        &mut PlainOldULE::<4>::from_byte_slice_unchecked_mut(data)[0]
+        #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
+        &mut RawBytesULE::<4>::from_byte_slice_unchecked_mut(data)[0]
     }
 
     /// Shift the indices starting with and after `starting_index` by the provided `amount`.
@@ -187,8 +180,10 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
     /// The length of the slice must be correctly set.
     unsafe fn shift_indices(&mut self, starting_index: usize, amount: i32) {
         let len = self.len();
+        #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
         let indices =
-            PlainOldULE::<4>::from_byte_slice_unchecked_mut(&mut self.entire_slice[4..4 + 4 * len]);
+            RawBytesULE::<4>::from_byte_slice_unchecked_mut(&mut self.entire_slice[4..4 + 4 * len]);
+        #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
         for idx in &mut indices[starting_index..] {
             *idx = (u32::from_unaligned(*idx).wrapping_add(amount as u32)).into();
         }
@@ -199,7 +194,7 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
     /// If you wish to repeatedly call methods on this [`VarZeroVecOwned`],
     /// it is more efficient to perform this conversion first
     pub fn as_varzerovec<'a>(&'a self) -> VarZeroVec<'a, T> {
-        self.as_borrowed().into()
+        self.as_slice().into()
     }
 
     /// Empty the vector
@@ -207,15 +202,10 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
         self.entire_slice.clear()
     }
 
-    /// Convert this vector to a regular vector of boxed DSTs
-    pub fn to_vec(&self) -> Vec<Box<T>> {
-        self.as_borrowed().to_vec()
-    }
-
-    /// Get a reference to the entire backing buffer of this vector
+    /// Consume this vector and return the backing buffer
     #[inline]
-    pub fn entire_slice(&self) -> &[u8] {
-        &self.entire_slice
+    pub fn into_bytes(self) -> Vec<u8> {
+        self.entire_slice
     }
 
     /// Invalidate and resize the data at an index, optionally inserting or removing the index.
@@ -258,6 +248,7 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
             new_size as i64 - (prev_element.end - prev_element.start) as i64 + index_shift;
         let new_slice_len = slice_len.wrapping_add(shift as usize);
         if shift > 0 {
+            #[allow(clippy::panic)] // TODO(#1668) Clippy exceptions need docs or fixing.
             if new_slice_len > u32::MAX as usize {
                 panic!(
                     "Attempted to grow VarZeroVec to an encoded size that does not fit within a u32"
@@ -330,6 +321,7 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
 
         // Return a mut slice to the new element data.
         let element_pos = 4 + self.len() * 4 + self.element_position_unchecked(index);
+        #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
         &mut self.entire_slice[element_pos..element_pos + new_size as usize]
     }
 
@@ -349,15 +341,16 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
             _ => (),
         }
         let len = unsafe {
+            #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
             u32::from_unaligned(
-                PlainOldULE::<4>::from_byte_slice_unchecked(&self.entire_slice[..4])[0],
+                RawBytesULE::<4>::from_byte_slice_unchecked(&self.entire_slice[..4])[0],
             )
         };
         if len == 0 {
             // An empty vec must have an empty slice: there is only a single valid byte representation.
             return false;
         }
-        if slice_len <= 4 + len as usize * 4 {
+        if slice_len < 4 + len as usize * 4 {
             // Not enough room for the indices.
             return false;
         }
@@ -370,7 +363,8 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
 
         // Test index validity.
         let indices = unsafe {
-            PlainOldULE::<4>::from_byte_slice_unchecked(&self.entire_slice[4..4 + len as usize * 4])
+            #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
+            RawBytesULE::<4>::from_byte_slice_unchecked(&self.entire_slice[4..4 + len as usize * 4])
         };
         for idx in indices {
             if u32::from_unaligned(*idx) > data_len {
@@ -379,6 +373,7 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
             }
         }
         for window in indices.windows(2) {
+            #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
             if u32::from_unaligned(window[0]) > u32::from_unaligned(window[1]) {
                 // Indices must be in non-decreasing order.
                 return false;
@@ -388,13 +383,14 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
     }
 
     /// Insert an element at the end of this vector
-    pub fn push<A: custom::EncodeAsVarULE<T> + ?Sized>(&mut self, element: &A) {
+    pub fn push<A: EncodeAsVarULE<T> + ?Sized>(&mut self, element: &A) {
         self.insert(self.len(), element)
     }
 
     /// Insert an element at index `idx`
-    pub fn insert<A: custom::EncodeAsVarULE<T> + ?Sized>(&mut self, index: usize, element: &A) {
+    pub fn insert<A: EncodeAsVarULE<T> + ?Sized>(&mut self, index: usize, element: &A) {
         let len = self.len();
+        #[allow(clippy::panic)] // TODO(#1668) Clippy exceptions need docs or fixing.
         if index > len {
             panic!(
                 "Called out-of-bounds insert() on VarZeroVec, index {} len {}",
@@ -409,13 +405,13 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
             self.reserve(8 + value_len);
             let len_u32 = 1u32;
             let index_u32 = 0u32;
-            self.entire_slice.extend(&len_u32.as_unaligned().0);
-            self.entire_slice.extend(&index_u32.as_unaligned().0);
-            element.encode_var_ule_as_slices(|slices| {
-                for slice in slices {
-                    self.entire_slice.extend(*slice)
-                }
-            });
+            self.entire_slice.extend(&len_u32.to_unaligned().0);
+            self.entire_slice.extend(&index_u32.to_unaligned().0);
+            let header_len = self.entire_slice.len();
+            self.entire_slice.resize(header_len + value_len, 0);
+            #[allow(clippy::indexing_slicing)]
+            // TODO(#1668) Clippy exceptions need docs or fixing.
+            element.encode_var_ule_write(&mut self.entire_slice[header_len..]);
             return;
         }
 
@@ -429,6 +425,7 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
     /// Remove the element at index `idx`
     pub fn remove(&mut self, index: usize) {
         let len = self.len();
+        #[allow(clippy::panic)] // TODO(#1668) Clippy exceptions need docs or fixing.
         if index >= len {
             panic!(
                 "Called out-of-bounds remove() on VarZeroVec, index {} len {}",
@@ -446,8 +443,9 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
     }
 
     /// Replace the element at index `idx` with another
-    pub fn replace<A: custom::EncodeAsVarULE<T> + ?Sized>(&mut self, index: usize, element: &A) {
+    pub fn replace<A: EncodeAsVarULE<T> + ?Sized>(&mut self, index: usize, element: &A) {
         let len = self.len();
+        #[allow(clippy::panic)] // TODO(#1668) Clippy exceptions need docs or fixing.
         if index >= len {
             panic!(
                 "Called out-of-bounds replace() on VarZeroVec, index {} len {}",
@@ -465,35 +463,12 @@ impl<T: VarULE + ?Sized> VarZeroVecOwned<T> {
     }
 }
 
-impl<T> VarZeroVecOwned<T>
-where
-    T: VarULE,
-    T: ?Sized,
-    T: Ord,
-{
-    /// Binary searches a sorted `VarZeroVecOwned<T>` for the given element. FoGeneralr more information, see
-    /// the primitive function [`binary_search`].
-    ///
-    /// [`binary_search`]: https://doc.rust-lang.org/std/primitive.slice.html#method.binary_search
-    #[inline]
-    pub fn binary_search(&self, x: &T) -> Result<usize, usize> {
-        self.as_borrowed().binary_search(x)
-    }
-}
-
-impl<T: VarULE + ?Sized> Index<usize> for VarZeroVecOwned<T> {
-    type Output = T;
-    fn index(&self, index: usize) -> &Self::Output {
-        self.get(index).expect("Indexing VarZeroVec out of bounds")
-    }
-}
-
 impl<T: VarULE + ?Sized> fmt::Debug for VarZeroVecOwned<T>
 where
     T: fmt::Debug,
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_list().entries(self.iter()).finish()
+        VarZeroSlice::fmt(self, f)
     }
 }
 
@@ -515,9 +490,9 @@ where
     }
 }
 
-impl<'a, T: ?Sized + VarULE> From<VarZeroVecBorrowed<'a, T>> for VarZeroVecOwned<T> {
-    fn from(other: VarZeroVecBorrowed<'a, T>) -> Self {
-        Self::from_borrowed(other)
+impl<'a, T: ?Sized + VarULE> From<&'a VarZeroSlice<T>> for VarZeroVecOwned<T> {
+    fn from(other: &'a VarZeroSlice<T>) -> Self {
+        Self::from_slice(other)
     }
 }
 
@@ -552,6 +527,30 @@ mod test {
         assert_eq!(zerovec, &*items);
     }
 
+    #[test]
+    // ensure that inserting empty items works
+    fn test_empty_inserts() {
+        let mut items: Vec<String> = Vec::new();
+        let mut zerovec = VarZeroVecOwned::<str>::new();
+
+        // Insert into an empty vec.
+        items.insert(0, "".into());
+        zerovec.insert(0, "");
+        assert_eq!(zerovec, &*items);
+
+        items.insert(0, "".into());
+        zerovec.insert(0, "");
+        assert_eq!(zerovec, &*items);
+
+        items.insert(0, "1234567890".into());
+        zerovec.insert(0, "1234567890");
+        assert_eq!(zerovec, &*items);
+
+        items.insert(0, "".into());
+        zerovec.insert(0, "");
+        assert_eq!(zerovec, &*items);
+    }
+
     #[test]
     fn test_small_insert_integrity() {
         // Tests that insert() works even when there
@@ -577,16 +576,8 @@ mod test {
 
     #[test]
     fn test_remove_integrity() {
-        let mut items: Vec<String> = vec![
-            "apples".into(),
-            "bananas".into(),
-            "eeples".into(),
-            "".into(),
-            "baneenees".into(),
-            "five".into(),
-            "".into(),
-        ];
-        let mut zerovec = VarZeroVecOwned::<str>::from_elements(&items);
+        let mut items: Vec<&str> = vec!["apples", "bananas", "eeples", "", "baneenees", "five", ""];
+        let mut zerovec = VarZeroVecOwned::<str>::try_from_elements(&items).unwrap();
 
         for index in [0, 2, 4, 0, 1, 1, 0] {
             items.remove(index);
@@ -597,10 +588,10 @@ mod test {
 
     #[test]
     fn test_removing_last_element_clears() {
-        let mut zerovec = VarZeroVecOwned::<str>::from_elements(&["buy some apples".to_string()]);
-        assert!(!zerovec.as_borrowed().entire_slice().is_empty());
+        let mut zerovec = VarZeroVecOwned::<str>::try_from_elements(&["buy some apples"]).unwrap();
+        assert!(!zerovec.as_bytes().is_empty());
         zerovec.remove(0);
-        assert!(zerovec.as_borrowed().entire_slice().is_empty());
+        assert!(zerovec.as_bytes().is_empty());
     }
 
     #[test]
@@ -611,39 +602,31 @@ mod test {
 
     #[test]
     fn test_replace_integrity() {
-        let mut items: Vec<String> = vec![
-            "apples".into(),
-            "bananas".into(),
-            "eeples".into(),
-            "".into(),
-            "baneenees".into(),
-            "five".into(),
-            "".into(),
-        ];
-        let mut zerovec = VarZeroVecOwned::<str>::from_elements(&items);
+        let mut items: Vec<&str> = vec!["apples", "bananas", "eeples", "", "baneenees", "five", ""];
+        let mut zerovec = VarZeroVecOwned::<str>::try_from_elements(&items).unwrap();
 
         // Replace with an element of the same size (and the first element)
-        items[0] = "blablah".into();
+        items[0] = "blablah";
         zerovec.replace(0, "blablah");
         assert_eq!(zerovec, &*items);
 
         // Replace with a smaller element
-        items[1] = "twily".into();
+        items[1] = "twily";
         zerovec.replace(1, "twily");
         assert_eq!(zerovec, &*items);
 
         // Replace an empty element
-        items[3] = "aoeuidhtns".into();
+        items[3] = "aoeuidhtns";
         zerovec.replace(3, "aoeuidhtns");
         assert_eq!(zerovec, &*items);
 
         // Replace the last element
-        items[6] = "0123456789".into();
+        items[6] = "0123456789";
         zerovec.replace(6, "0123456789");
         assert_eq!(zerovec, &*items);
 
         // Replace with an empty element
-        items[2] = "".into();
+        items[2] = "";
         zerovec.replace(2, "");
         assert_eq!(zerovec, &*items);
     }
