@@ -2,13 +2,14 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
-use super::{VarZeroVec, VarZeroVecBorrowed};
+use super::{VarZeroSlice, VarZeroVec};
 use crate::ule::*;
 use alloc::boxed::Box;
 use alloc::vec::Vec;
 use core::fmt;
 use core::marker::PhantomData;
 use serde::de::{self, Deserialize, Deserializer, SeqAccess, Visitor};
+#[cfg(feature = "serde_serialize")]
 use serde::ser::{Serialize, SerializeSeq, Serializer};
 
 struct VarZeroVecVisitor<T: ?Sized> {
@@ -53,7 +54,7 @@ where
         while let Some(value) = seq.next_element::<Box<T>>()? {
             vec.push(value);
         }
-        Ok((&*vec).into())
+        Ok(VarZeroVec::from(&vec))
     }
 }
 
@@ -78,7 +79,7 @@ where
 }
 
 /// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
-impl<'de, 'a, T> Deserialize<'de> for VarZeroVecBorrowed<'a, T>
+impl<'de, 'a, T> Deserialize<'de> for &'a VarZeroSlice<T>
 where
     T: VarULE + ?Sized,
     Box<T>: Deserialize<'de>,
@@ -90,7 +91,7 @@ where
     {
         if deserializer.is_human_readable() {
             Err(de::Error::custom(
-                "VarZeroVecBorrowed cannot be deserialized from human-readable formats",
+                "&VarZeroSlice cannot be deserialized from human-readable formats",
             ))
         } else {
             let deserialized: VarZeroVec<'a, T> = VarZeroVec::deserialize(deserializer)?;
@@ -98,7 +99,7 @@ where
                 b
             } else {
                 return Err(de::Error::custom(
-                    "VarZeroVecBorrowed can only deserialize in zero-copy ways",
+                    "&VarZeroSlice can only deserialize in zero-copy ways",
                 ));
             };
             Ok(borrowed)
@@ -107,6 +108,7 @@ where
 }
 
 /// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
+#[cfg(feature = "serde_serialize")]
 impl<T> Serialize for VarZeroVec<'_, T>
 where
     T: Serialize + VarULE + ?Sized,
@@ -122,13 +124,14 @@ where
             }
             seq.end()
         } else {
-            serializer.serialize_bytes(self.get_encoded_slice())
+            serializer.serialize_bytes(self.as_bytes())
         }
     }
 }
 
 /// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
-impl<T> Serialize for VarZeroVecBorrowed<'_, T>
+#[cfg(feature = "serde_serialize")]
+impl<T> Serialize for VarZeroSlice<T>
 where
     T: Serialize + VarULE + ?Sized,
 {
@@ -136,25 +139,25 @@ where
     where
         S: Serializer,
     {
-        VarZeroVec::from(*self).serialize(serializer)
+        self.as_varzerovec().serialize(serializer)
     }
 }
 
 #[cfg(test)]
 #[allow(non_camel_case_types)]
 mod test {
-    use super::super::*;
+    use crate::{VarZeroSlice, VarZeroVec};
 
-    #[derive(::serde::Serialize, ::serde::Deserialize)]
+    #[derive(serde::Serialize, serde::Deserialize)]
     struct DeriveTest_VarZeroVec<'data> {
         #[serde(borrow)]
         _data: VarZeroVec<'data, str>,
     }
 
-    #[derive(::serde::Serialize, ::serde::Deserialize)]
-    struct DeriveTest_VarZeroVecBorrowed<'data> {
+    #[derive(serde::Serialize, serde::Deserialize)]
+    struct DeriveTest_VarZeroSlice<'data> {
         #[serde(borrow)]
-        _data: VarZeroVecBorrowed<'data, str>,
+        _data: &'data VarZeroSlice<str>,
     }
 
     // ["foo", "bar", "baz", "dolor", "quux", "lorem ipsum"];
@@ -204,12 +207,12 @@ mod test {
 
     #[test]
     fn test_vzv_borrowed() {
-        let zerovec_orig: VarZeroVecBorrowed<str> =
-            VarZeroVecBorrowed::parse_byte_slice(BYTES).expect("parse");
+        let zerovec_orig: &VarZeroSlice<str> =
+            VarZeroSlice::parse_byte_slice(BYTES).expect("parse");
         let bincode_buf = bincode::serialize(&zerovec_orig).expect("serialize");
         assert_eq!(BINCODE_BUF, bincode_buf);
-        let zerovec_new: VarZeroVecBorrowed<str> = bincode::deserialize(&bincode_buf)
-            .expect("deserialize from buffer to VarZeroVecBorrowed");
+        let zerovec_new: &VarZeroSlice<str> =
+            bincode::deserialize(&bincode_buf).expect("deserialize from buffer to VarZeroSlice");
         assert_eq!(zerovec_orig.to_vec(), zerovec_new.to_vec());
     }
 
