@@ -5,10 +5,16 @@
 #[cfg(feature = "serde")]
 mod serde;
 
+mod slice;
+
+pub use slice::ZeroSlice;
+
 use crate::ule::*;
 use alloc::vec::Vec;
+use core::cmp::{Ord, Ordering, PartialOrd};
 use core::fmt;
 use core::iter::FromIterator;
+use core::ops::Deref;
 
 /// A zero-copy vector for fixed-width types.
 ///
@@ -16,7 +22,49 @@ use core::iter::FromIterator;
 /// desirable to borrow data from an unaligned byte slice, such as zero-copy deserialization.
 ///
 /// `T` must implement [`AsULE`], which is auto-implemented for a number of built-in types,
-/// including all fixed-width multibyte integers.
+/// including all fixed-width multibyte integers. For variable-width types like [`str`],
+/// see [`VarZeroVec`](crate::VarZeroVec). [`zerovec::make_ule`](crate::make_ule) may
+/// be used to automatically implement [`AsULE`] for a type and generate the underlying [`ULE`] type.
+///
+/// Typically, the zero-copy equivalent of a `Vec<T>` will simply be `ZeroVec<'a, T>`.
+///
+/// Most of the methods on `ZeroVec<'a, T>` come from its [`Deref`] implementation to [`ZeroSlice<T>`](ZeroSlice).
+///
+/// For creating zero-copy vectors of fixed-size types, see [`VarZeroVec`](crate::VarZeroVec).
+///
+/// `ZeroVec<T>` behaves much like [`Cow`](alloc::borrow::Cow), where it can be constructed from
+/// owned data (and then mutated!) but can also borrow from some buffer.
+///
+/// # Example
+///
+/// ```
+/// use zerovec::ZeroVec;
+///
+/// // The little-endian bytes correspond to the numbers on the following line.
+/// let nums: &[u16] = &[211, 281, 421, 461];
+///
+/// #[derive(serde::Serialize, serde::Deserialize)]
+/// struct Data<'a> {
+///     #[serde(borrow)]
+///     nums: ZeroVec<'a, u16>,
+/// }
+///
+/// // The owned version will allocate
+/// let data = Data { nums: ZeroVec::alloc_from_slice(nums) };
+/// let bincode_bytes = bincode::serialize(&data)
+///     .expect("Serialization should be successful");
+///
+/// // Will deserialize without allocations
+/// let deserialized: Data = bincode::deserialize(&bincode_bytes)
+///     .expect("Deserialization should be successful");
+///
+/// // This deserializes without allocation!
+/// assert!(matches!(deserialized.nums, ZeroVec::Borrowed(_)));
+/// assert_eq!(deserialized.nums.get(2), Some(421));
+/// assert_eq!(deserialized.nums, nums);
+/// ```
+///
+/// [`ule`]: crate::ule
 ///
 /// # How it Works
 ///
@@ -25,47 +73,20 @@ use core::iter::FromIterator;
 /// items from `ZeroVec<T>`, we fetch the `T::ULE`, convert it on the fly to `T`, and return `T` by
 /// value.
 ///
-/// Benchmarks can be found in the project repository. We found that for common operations on small
-/// and large vectors, `ZeroVec<T>` performs from slightly faster to 15% slower than `Vec<T>`.
-/// However, the main performance improvement on `ZeroVec<T>` is when deserializing from a byte
-/// array; `ZeroVec<T>` deserializes 80% faster than `Vec<T>` in Serde Bincode, and it does not
-/// require any heap allocations.
+/// Benchmarks can be found in the project repository, with some results found in the [crate-level documentation](crate).
 ///
-/// # Safety
-///
-/// `ZeroVec<T>` contains no unsafe code. However, the conversion from `&[u8]` to `&[T::ULE]` may
-/// be unsafe. For more information, see the [`ule`] module.
-///
-/// # Example
-///
-/// ```
-/// use zerovec::ZeroVec;
-///
-/// // The little-endian bytes correspond to the numbers on the following line.
-/// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x01];
-/// let nums: &[u16] = &[211, 281, 421, 461];
-///
-/// // Conversion from &[u8] to &[u16::ULE] is infallible.
-/// let zerovec: ZeroVec<u16> = ZeroVec::parse_byte_slice(bytes).expect("infallible");
-///
-/// assert!(matches!(zerovec, ZeroVec::Borrowed(_)));
-/// assert_eq!(zerovec.get(2), Some(421));
-/// assert_eq!(zerovec, nums);
-/// ```
-///
-/// [`ule`]: crate::ule
-#[non_exhaustive]
+/// See [the design doc](https://github.com/unicode-org/icu4x/blob/main/utils/zerovec/design_doc.md) for more details.
 #[derive(Clone)]
 pub enum ZeroVec<'a, T>
 where
     T: AsULE + ?Sized,
 {
-    /// An owned `ZeroVec<T>`. This will typically be constructed by [`ZeroVec::clone_from_slice()`]
+    /// An owned `ZeroVec<T>`. This will typically be constructed by [`ZeroVec::alloc_from_slice()`]
     /// or by calling [`ZeroVec::to_mut()`]/[`ZeroVec::for_each_mut()`]/etc on [`ZeroVec::Borrowed`].
     Owned(Vec<T::ULE>),
 
     /// A borrowed `ZeroVec<T>`. This will typically be constructed by [`ZeroVec::parse_byte_slice()`],
-    /// [`ZeroVec::from_slice()`], or deserializers capable of doing zero-copy deserialization.
+    /// [`ZeroVec::from_slice_or_alloc()`], or deserializers capable of doing zero-copy deserialization.
     ///
     /// If you already have a slice of `[T::ULE]`s, you can directly construct one of these.
     ///
@@ -77,8 +98,8 @@ where
     ///
     /// // The little-endian bytes correspond to the numbers on the following line.
     /// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x01];
-    /// let nums: &[PlainOldULE<2>] = &[211_u16.as_unaligned(), 281_u16.as_unaligned(),
-    ///                                 421_u16.as_unaligned(), 461_u16.as_unaligned()];
+    /// let nums: &[RawBytesULE<2>] = &[211_u16.to_unaligned(), 281_u16.to_unaligned(),
+    ///                                 421_u16.to_unaligned(), 461_u16.to_unaligned()];
     ///
     /// let zerovec = ZeroVec::<u16>::Borrowed(nums);
     ///
@@ -88,6 +109,23 @@ where
     Borrowed(&'a [T::ULE]),
 }
 
+impl<'a, T: AsULE> Deref for ZeroVec<'a, T> {
+    type Target = ZeroSlice<T>;
+    fn deref(&self) -> &Self::Target {
+        let slice = match self {
+            ZeroVec::Owned(vec) => &**vec,
+            ZeroVec::Borrowed(slice) => *slice,
+        };
+        ZeroSlice::from_ule_slice(slice)
+    }
+}
+
+impl<'a, T: AsULE> AsRef<ZeroSlice<T>> for ZeroVec<'a, T> {
+    fn as_ref(&self) -> &ZeroSlice<T> {
+        self.deref()
+    }
+}
+
 impl<T> fmt::Debug for ZeroVec<'_, T>
 where
     T: AsULE + fmt::Debug,
@@ -123,6 +161,16 @@ where
     }
 }
 
+impl<T, const N: usize> PartialEq<[T; N]> for ZeroVec<'_, T>
+where
+    T: AsULE + PartialEq + ?Sized,
+{
+    #[inline]
+    fn eq(&self, other: &[T; N]) -> bool {
+        self.iter().eq(other.iter().copied())
+    }
+}
+
 impl<'a, T: AsULE> Default for ZeroVec<'a, T> {
     #[inline]
     fn default() -> Self {
@@ -130,12 +178,24 @@ impl<'a, T: AsULE> Default for ZeroVec<'a, T> {
     }
 }
 
+impl<'a, T: AsULE + PartialOrd> PartialOrd for ZeroVec<'a, T> {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        self.iter().partial_cmp(other.iter())
+    }
+}
+
+impl<'a, T: AsULE + Ord> Ord for ZeroVec<'a, T> {
+    fn cmp(&self, other: &Self) -> Ordering {
+        self.iter().cmp(other.iter())
+    }
+}
+
 impl<'a, T> ZeroVec<'a, T>
 where
     T: AsULE + ?Sized,
 {
     #[inline]
-    /// Creates a new, empty `ZeroVec<T>`.
+    /// Creates a new, borrowed, empty `ZeroVec<T>`.
     ///
     /// # Examples
     ///
@@ -149,6 +209,11 @@ where
         Self::Borrowed(&[])
     }
 
+    /// Creates a new, owned, empty `ZeroVec<T>`, with a certain capacity pre-allocated.
+    pub fn with_capacity(capacity: usize) -> Self {
+        Self::Owned(Vec::with_capacity(capacity))
+    }
+
     /// Parses a `&[u8]` buffer into a `ZeroVec<T>`.
     ///
     /// This function is infallible for built-in integer types, but fallible for other types,
@@ -172,84 +237,11 @@ where
     /// assert!(matches!(zerovec, ZeroVec::Borrowed(_)));
     /// assert_eq!(zerovec.get(2), Some(421));
     /// ```
-    pub fn parse_byte_slice(bytes: &'a [u8]) -> Result<Self, <<T as AsULE>::ULE as ULE>::Error> {
+    pub fn parse_byte_slice(bytes: &'a [u8]) -> Result<Self, ZeroVecError> {
         let slice: &'a [T::ULE] = T::ULE::parse_byte_slice(bytes)?;
         Ok(Self::Borrowed(slice))
     }
 
-    /// Returns a `ZeroVec<T>` as its underlying `&[u8]` byte buffer representation.
-    ///
-    /// Useful for serialization.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use zerovec::ZeroVec;
-    ///
-    /// // The little-endian bytes correspond to the numbers on the following line.
-    /// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x01];
-    /// let nums: &[u16] = &[211, 281, 421, 461];
-    ///
-    /// let zerovec = ZeroVec::clone_from_slice(nums);
-    ///
-    /// assert_eq!(bytes, zerovec.as_bytes());
-    /// ```
-    #[inline]
-    pub fn as_bytes(&self) -> &[u8] {
-        T::ULE::as_byte_slice(self.as_slice())
-    }
-
-    /// Dereferences this `ZeroVec<T>` as `&[T::ULE]`. Most other functions on `ZeroVec<T>` use
-    /// this function as a building block.
-    #[inline]
-    pub fn as_slice(&self) -> &[T::ULE] {
-        match self {
-            Self::Owned(vec) => vec.as_slice(),
-            Self::Borrowed(slice) => slice,
-        }
-    }
-
-    /// Returns the number of elements in this `ZeroVec<T>`.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use zerovec::ZeroVec;
-    /// use zerovec::ule::AsULE;
-    ///
-    /// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x01];
-    /// let zerovec: ZeroVec<u16> = ZeroVec::parse_byte_slice(bytes).expect("infallible");
-    ///
-    /// assert_eq!(4, zerovec.len());
-    /// assert_eq!(
-    ///     bytes.len(),
-    ///     zerovec.len() * std::mem::size_of::<<u16 as AsULE>::ULE>()
-    /// );
-    /// ```
-    #[inline]
-    pub fn len(&self) -> usize {
-        self.as_slice().len()
-    }
-
-    /// Returns whether the vec is empty.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use zerovec::ZeroVec;
-    ///
-    /// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x01];
-    /// let zerovec: ZeroVec<u16> = ZeroVec::parse_byte_slice(bytes).expect("infallible");
-    /// assert!(!zerovec.is_empty());
-    ///
-    /// let emptyvec: ZeroVec<u16> = ZeroVec::parse_byte_slice(&[]).expect("infallible");
-    /// assert!(emptyvec.is_empty());
-    /// ```
-    #[inline]
-    pub fn is_empty(&self) -> bool {
-        self.as_slice().is_empty()
-    }
-
     /// Converts a `ZeroVec<T>` into a `ZeroVec<u8>`, retaining the current ownership model.
     ///
     /// Note that the length of the ZeroVec may change.
@@ -275,7 +267,7 @@ where
     /// use zerovec::ZeroVec;
     ///
     /// let nums: &[u16] = &[211, 281, 421, 461];
-    /// let zerovec = ZeroVec::clone_from_slice(nums);
+    /// let zerovec = ZeroVec::alloc_from_slice(nums);
     /// let zv_bytes = zerovec.into_bytes();
     ///
     /// assert!(matches!(zv_bytes, ZeroVec::Owned(_)));
@@ -294,8 +286,40 @@ where
         }
     }
 
+    /// Casts a `ZeroVec<T>` to a compatible `ZeroVec<P>`.
+    ///
+    /// `T` and `P` are compatible if they have the same `ULE` representation.
+    ///
+    /// If the `ULE`s of `T` and `P` are different types but have the same size,
+    /// use [`Self::try_into_converted()`].
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use zerovec::ZeroVec;
+    ///
+    /// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x80];
+    ///
+    /// let zerovec_u16: ZeroVec<u16> = ZeroVec::parse_byte_slice(bytes).expect("infallible");
+    /// assert_eq!(zerovec_u16.get(3), Some(32973));
+    ///
+    /// let zerovec_i16: ZeroVec<i16> = zerovec_u16.cast();
+    /// assert_eq!(zerovec_i16.get(3), Some(-32563));
+    /// ```
+    pub fn cast<P>(self) -> ZeroVec<'a, P>
+    where
+        P: AsULE<ULE = T::ULE>,
+    {
+        match self {
+            Self::Owned(v) => ZeroVec::Owned(v),
+            Self::Borrowed(v) => ZeroVec::Borrowed(v),
+        }
+    }
+
     /// Converts a `ZeroVec<T>` into a `ZeroVec<P>`, retaining the current ownership model.
     ///
+    /// If `T` and `P` have the exact same `ULE`, use [`Self::cast()`].
+    ///
     /// # Panics
     ///
     /// Panics if `T::ULE` and `P::ULE` are not the same size.
@@ -323,7 +347,7 @@ where
     /// use zerovec::ZeroVec;
     ///
     /// let chars: &[char] = &['🍿', '🙉'];
-    /// let zv_char = ZeroVec::clone_from_slice(chars);
+    /// let zv_char = ZeroVec::alloc_from_slice(chars);
     /// let zv_u32: ZeroVec<u32> = zv_char.try_into_converted()
     ///     .expect("length is divisible");
     ///
@@ -358,7 +382,7 @@ where
     /// assert!(matches!(zv_u16, ZeroVec::Borrowed(_)));
     /// assert_eq!(zv_u16.get(0), Some(0xF37F));
     /// ```
-    pub fn try_into_converted<P: AsULE>(self) -> Result<ZeroVec<'a, P>, <P::ULE as ULE>::Error> {
+    pub fn try_into_converted<P: AsULE>(self) -> Result<ZeroVec<'a, P>, ZeroVecError> {
         assert_eq!(
             core::mem::size_of::<<T as AsULE>::ULE>(),
             core::mem::size_of::<<P as AsULE>::ULE>()
@@ -426,7 +450,7 @@ impl<'a> ZeroVec<'a, u8> {
     /// assert!(matches!(zerovec, ZeroVec::Owned(_)));
     /// assert_eq!(zerovec.get(0), Some(211));
     /// ```
-    pub fn try_into_parsed<T: AsULE>(self) -> Result<ZeroVec<'a, T>, <T::ULE as ULE>::Error> {
+    pub fn try_into_parsed<T: AsULE>(self) -> Result<ZeroVec<'a, T>, ZeroVecError> {
         match self {
             ZeroVec::Borrowed(bytes) => {
                 let slice: &'a [T::ULE] = T::ULE::parse_byte_slice(bytes)?;
@@ -457,14 +481,14 @@ where
     /// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x01];
     /// let nums: &[u16] = &[211, 281, 421, 461];
     ///
-    /// let zerovec = ZeroVec::clone_from_slice(nums);
+    /// let zerovec = ZeroVec::alloc_from_slice(nums);
     ///
     /// assert!(matches!(zerovec, ZeroVec::Owned(_)));
     /// assert_eq!(bytes, zerovec.as_bytes());
     /// ```
     #[inline]
-    pub fn clone_from_slice(other: &[T]) -> Self {
-        Self::Owned(other.iter().copied().map(T::as_unaligned).collect())
+    pub fn alloc_from_slice(other: &[T]) -> Self {
+        Self::Owned(other.iter().copied().map(T::to_unaligned).collect())
     }
 
     /// Creates a `Vec<T>` from a `ZeroVec<T>`.
@@ -475,23 +499,19 @@ where
     /// use zerovec::ZeroVec;
     ///
     /// let nums: &[u16] = &[211, 281, 421, 461];
-    /// let vec: Vec<u16> = ZeroVec::clone_from_slice(nums).to_vec();
+    /// let vec: Vec<u16> = ZeroVec::alloc_from_slice(nums).to_vec();
     ///
     /// assert_eq!(nums, vec.as_slice());
     /// ```
     #[inline]
     pub fn to_vec(&self) -> Vec<T> {
-        self.as_slice()
-            .iter()
-            .copied()
-            .map(T::from_unaligned)
-            .collect()
+        self.iter().collect()
     }
 }
 
 impl<'a, T> ZeroVec<'a, T>
 where
-    T: AsULE + SliceAsULE,
+    T: EqULE,
 {
     /// Attempts to create a `ZeroVec<'a, T>` from a `&'a [T]` by borrowing the argument.
     ///
@@ -513,7 +533,7 @@ where
     /// ```
     #[inline]
     pub fn try_from_slice(slice: &'a [T]) -> Option<Self> {
-        T::slice_as_unaligned(slice).map(|ule_slice| Self::Borrowed(ule_slice))
+        T::slice_to_unaligned(slice).map(|ule_slice| Self::Borrowed(ule_slice))
     }
 
     /// Creates a `ZeroVec<'a, T>` from a `&'a [T]`, either by borrowing the argument or by
@@ -531,14 +551,14 @@ where
     /// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x01];
     /// let nums: &[u16] = &[211, 281, 421, 461];
     ///
-    /// let zerovec = ZeroVec::from_slice(nums);
+    /// let zerovec = ZeroVec::from_slice_or_alloc(nums);
     ///
     /// // Note: zerovec could be either borrowed or owned.
     /// assert_eq!(bytes, zerovec.as_bytes());
     /// ```
     #[inline]
-    pub fn from_slice(slice: &'a [T]) -> Self {
-        Self::try_from_slice(slice).unwrap_or_else(|| Self::clone_from_slice(slice))
+    pub fn from_slice_or_alloc(slice: &'a [T]) -> Self {
+        Self::try_from_slice(slice).unwrap_or_else(|| Self::alloc_from_slice(slice))
     }
 }
 
@@ -546,84 +566,6 @@ impl<T> ZeroVec<'_, T>
 where
     T: AsULE,
 {
-    /// Gets the element at the specified index. Returns None if out of range.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use zerovec::ZeroVec;
-    ///
-    /// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x01];
-    /// let zerovec: ZeroVec<u16> = ZeroVec::parse_byte_slice(bytes).expect("infallible");
-    ///
-    /// assert_eq!(zerovec.get(2), Some(421));
-    /// assert_eq!(zerovec.get(4), None);
-    /// ```
-    #[inline]
-    pub fn get(&self, index: usize) -> Option<T> {
-        self.as_slice().get(index).copied().map(T::from_unaligned)
-    }
-
-    pub(crate) fn get_ule_ref(&self, index: usize) -> Option<&T::ULE> {
-        self.as_slice().get(index)
-    }
-
-    /// Gets the first element. Returns None if empty.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use zerovec::ZeroVec;
-    ///
-    /// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x01];
-    /// let zerovec: ZeroVec<u16> = ZeroVec::parse_byte_slice(bytes).expect("infallible");
-    ///
-    /// assert_eq!(zerovec.first(), Some(211));
-    /// ```
-    #[inline]
-    pub fn first(&self) -> Option<T> {
-        self.as_slice().first().copied().map(T::from_unaligned)
-    }
-
-    /// Gets the last element. Returns None if empty.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use zerovec::ZeroVec;
-    ///
-    /// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x01];
-    /// let zerovec: ZeroVec<u16> = ZeroVec::parse_byte_slice(bytes).expect("infallible");
-    ///
-    /// assert_eq!(zerovec.last(), Some(461));
-    /// ```
-    #[inline]
-    pub fn last(&self) -> Option<T> {
-        self.as_slice().last().copied().map(T::from_unaligned)
-    }
-
-    /// Gets an iterator over the elements.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use zerovec::ZeroVec;
-    ///
-    /// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x01];
-    /// let zerovec: ZeroVec<u16> = ZeroVec::parse_byte_slice(bytes).expect("infallible");
-    /// let mut it = zerovec.iter();
-    ///
-    /// assert_eq!(it.next(), Some(211));
-    /// assert_eq!(it.next(), Some(281));
-    /// assert_eq!(it.next(), Some(421));
-    /// assert_eq!(it.next(), Some(461));
-    /// assert_eq!(it.next(), None);
-    /// ```
-    #[inline]
-    pub fn iter(&self) -> impl Iterator<Item = T> + '_ {
-        self.as_slice().iter().copied().map(T::from_unaligned)
-    }
-
     /// Mutates each element according to a given function, meant to be
     /// a more convenient version of calling `.iter_mut()` on
     /// [`ZeroVec::to_mut()`] which serves fewer use cases.
@@ -649,7 +591,7 @@ where
         self.to_mut().iter_mut().for_each(|item| {
             let mut aligned = T::from_unaligned(*item);
             f(&mut aligned);
-            *item = aligned.as_unaligned()
+            *item = aligned.to_unaligned()
         });
     }
 
@@ -681,7 +623,7 @@ where
         self.to_mut().iter_mut().try_for_each(|item| {
             let mut aligned = T::from_unaligned(*item);
             f(&mut aligned)?;
-            *item = aligned.as_unaligned();
+            *item = aligned.to_unaligned();
             Ok(())
         })
     }
@@ -704,7 +646,7 @@ where
         match self {
             Self::Owned(vec) => ZeroVec::Owned(vec),
             Self::Borrowed(_) => {
-                let vec: Vec<T::ULE> = self.iter().map(T::as_unaligned).collect();
+                let vec: Vec<T::ULE> = self.iter().map(T::to_unaligned).collect();
                 ZeroVec::Owned(vec)
             }
         }
@@ -723,14 +665,14 @@ where
     /// let mut zerovec: ZeroVec<u16> = ZeroVec::parse_byte_slice(bytes).expect("infallible");
     /// assert!(matches!(zerovec, ZeroVec::Borrowed(_)));
     ///
-    /// zerovec.to_mut().push(12_u16.as_unaligned());
+    /// zerovec.to_mut().push(12_u16.to_unaligned());
     /// assert!(matches!(zerovec, ZeroVec::Owned(_)));
     /// ```
     pub fn to_mut(&mut self) -> &mut Vec<T::ULE> {
         match self {
             ZeroVec::Owned(ref mut vec) => vec,
             ZeroVec::Borrowed(_) => {
-                let vec: Vec<T::ULE> = self.iter().map(T::as_unaligned).collect();
+                let vec: Vec<T::ULE> = self.iter().map(T::to_unaligned).collect();
                 let new_self = ZeroVec::Owned(vec);
                 *self = new_self;
                 // recursion is limited since we are guaranteed to hit the Owned branch
@@ -738,32 +680,10 @@ where
             }
         }
     }
-}
 
-impl<T> ZeroVec<'_, T>
-where
-    T: AsULE + Ord,
-{
-    /// Binary searches a sorted `ZeroVec<T>` for the given element. For more information, see
-    /// the primitive function [`binary_search`].
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use zerovec::ZeroVec;
-    ///
-    /// let bytes: &[u8] = &[0xD3, 0x00, 0x19, 0x01, 0xA5, 0x01, 0xCD, 0x01];
-    /// let zerovec: ZeroVec<u16> = ZeroVec::parse_byte_slice(bytes).expect("infallible");
-    ///
-    /// assert_eq!(zerovec.binary_search(&281), Ok(1));
-    /// assert_eq!(zerovec.binary_search(&282), Err(2));
-    /// ```
-    ///
-    /// [`binary_search`]: https://doc.rust-lang.org/std/primitive.slice.html#method.binary_search
-    #[inline]
-    pub fn binary_search(&self, x: &T) -> Result<usize, usize> {
-        self.as_slice()
-            .binary_search_by(|probe| T::from_unaligned(*probe).cmp(x))
+    /// Remove all elements from this ZeroVec and reset it to an empty borrowed state.
+    pub fn clear(&mut self) {
+        *self = Self::Borrowed(&[])
     }
 }
 
@@ -773,7 +693,7 @@ impl<T: AsULE> FromIterator<T> for ZeroVec<'_, T> {
     where
         I: IntoIterator<Item = T>,
     {
-        ZeroVec::Owned(iter.into_iter().map(|t| t.as_unaligned()).collect())
+        ZeroVec::Owned(iter.into_iter().map(|t| t.to_unaligned()).collect())
     }
 }
 
@@ -785,7 +705,7 @@ mod tests {
     #[test]
     fn test_get() {
         {
-            let zerovec = ZeroVec::from_slice(TEST_SLICE);
+            let zerovec = ZeroVec::from_slice_or_alloc(TEST_SLICE);
             assert_eq!(zerovec.get(0), Some(TEST_SLICE[0]));
             assert_eq!(zerovec.get(1), Some(TEST_SLICE[1]));
             assert_eq!(zerovec.get(2), Some(TEST_SLICE[2]));
@@ -801,7 +721,7 @@ mod tests {
     #[test]
     fn test_binary_search() {
         {
-            let zerovec = ZeroVec::from_slice(TEST_SLICE);
+            let zerovec = ZeroVec::from_slice_or_alloc(TEST_SLICE);
             assert_eq!(Ok(3), zerovec.binary_search(&0x0e0d0c));
             assert_eq!(Err(3), zerovec.binary_search(&0x0c0d0c));
         }
@@ -868,7 +788,7 @@ mod tests {
                 .unwrap()
                 .get(19)
         );
-        // TODO(#1144): Check for correct slice length in PlainOldULE
+        // TODO(#1144): Check for correct slice length in RawBytesULE
         // assert_eq!(
         //     None,
         //     ZeroVec::<u32>::parse_byte_slice(&TEST_BUFFER_LE[77..])
